/*
  libsmtp is a library to send mail via SMTP
     This is the communication part
   
Copyright  2001 Kevin Read <obsidian@berlios.de>

This software is available under the GNU Lesser Public License as described
in the COPYING file.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Kevin Read <obsidian@berlios.de>
Thu Aug 16 2001 

The simple_sendmail function sends a simple mail using the library functions.
It uses fork() to separate from the main program.
Ali BASEL <alib@sabanciuniv.edu>
2003.05.10
*/

#include <glib.h>
#include "../config.h"
#include "libsmtp.h"
#include <sys/types.h>
#include <unistd.h>
#include <syslog.h>

int simple_sendmail(char *server, char *from, char *to, char *subject, char *message)
{
  int test_temp=0, pid;
	
  /* This struct holds all session data. You need one per mail server
     connection */
  struct libsmtp_session_struct *mailsession;
  
  /* We need these to look through the GLists later on */
  GList *temp_glist;
  GList *temp_glist_address;

  if( (pid=fork())<0) {
			syslog(LOG_ERR, "main fork() error %m");
			return(1);
  }
  if( !pid ) // for child process pid=0, for main process pid=number
  	return(0); //child process returns, main is going to send the mail in the background
 
  	//Starts the program in a new session
	setsid();
  
  /* This mallocs the structs mem and initializes variables in it */
  mailsession = libsmtp_session_initialize();
  
  /* Set session environment (from address, subject) */
  libsmtp_set_environment (from,subject, 0, mailsession);
  
  /* Now we add some recipients */
  libsmtp_add_recipient (LIBSMTP_REC_TO, to, mailsession);

  /* This starts the SMTP connection */
  if (libsmtp_connect (server,0,0,mailsession))
  {
    syslog(LOG_INFO,"An error occured while connecting:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
    exit(1);
  }
	  
  /* This will conduct the SMTP dialogue */
  if (libsmtp_dialogue (mailsession))
  {
    syslog(LOG_INFO,"An error occured while conducting the SMTP dialogue:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
    exit(1);
  }
    
  /* Now lets send the headers - you can send your own headers too */
  if (libsmtp_headers (mailsession))
  {
    syslog(LOG_INFO,"An error occured while sending header data:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
    exit(1);
  }
	  
  /* This sends a line of message body */
  if (libsmtp_body_send_raw (message, strlen(message), mailsession))
  {
    syslog(LOG_INFO,"An error occured while sending the body:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
    exit(1);
  }
	  
  /* This ends the body part */
  if (libsmtp_body_end (mailsession))
  {
    syslog(LOG_INFO,"An error occured while sending the body:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
    exit(1);
  }
    
  /* This ends the connection gracefully */
  if (libsmtp_quit (mailsession))
  {
    /* Actually there can't be an error here... :) */
    syslog(LOG_INFO,"An error occured while closing the connection:\n%s\nLast Response:%s\n", libsmtp_strerr (mailsession), mailsession->LastResponse->str);
	exit(1);
  }

  /* Free the allocated struct mem */
  free (mailsession);
  exit(0);
}
