#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fnmatch.h>
#include <unistd.h>
#include <signal.h>
#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>

#include "dosya_okuma.h"
#include "libsmtp.h"

#define MAXUZ 4096
#define MAXSTR 128
#define SMTPHOST "localhost"
#define FROM "Dikkat@sabanciuniv.edu"
#define OKSUBJECT "tail_mail OK"
#define CONFIGFILE "/etc/tail-mail.conf"
#define TIME1 8
#define TIME2 16
#define LOGFACILITY LOG_LOCAL1

void usage(void);
void cik(int sig);
int subject_to(char *string, char *subject, char *to);
void comma(char *buf);

int satir_sayisi;
char **satirlar;

void usage(void)
{
	puts("\ntail_mail looks for strings in the config file, for the given log file,");
	puts("and, informs by e-mail with the specified subjects.\n");
	puts("usage: tail-mail [-c config-file] -f log_to_check\n");
	puts("Ali BASEL <alib@sabanciuniv.edu> 2003.06.03");
	exit(0);
}
int main(int argc, char **argv)
{
	int op, i=0, j, pid, report=1; 
	char message[MAXUZ+1], to[MAXSTR];
	char subject[MAXSTR+1], configdosyasi[MAXSTR+1]=CONFIGFILE;
	char check_file[MAXSTR+1]="", buf1[MAXUZ+1], buf2[MAXSTR+1];
	time_t curtime;
	struct tm *tmb;
	FILE *dosya;	
	unsigned long int konum=0L, file_length;
	
	while( (op=getopt(argc, argv, "c:f:h?V"))!=-1)
		switch(op) {
			case 'c':
				strncpy(configdosyasi, optarg, MAXSTR);
				break;
			case 'f':
				strncpy(check_file, optarg, MAXSTR);
				break;
			default: usage();
		}
	
	openlog("tail_mail", LOG_NOWAIT,  LOGFACILITY);
	printf("Config file: \"%s\".\n", configdosyasi);
	if( check_file[0]==(char)NULL ) usage();
		
	if( !(satirlar=dosya_satirlarini_getir( configdosyasi, &satir_sayisi )) ) {
		printf("\"%s\" config file is empty !\n", configdosyasi);
		syslog(LOG_ERR, "\"%s\" config file is empty !", configdosyasi); exit(0);
	}
			
	printf("The number of strings to look for:%d\n", (satir_sayisi)/2);
	for(i=0; i<satir_sayisi; i+=2)
		printf("string[%d]:%s\n", i, satirlar[i]);

	if( !(dosya=fopen(check_file, "r")) ) {
			printf("File \"%s\" is not found !\n", check_file);
			cik(0); 
	}
	fseek(dosya, 0, SEEK_END); konum=ftell(dosya);
	fclose(dosya); 
#ifdef DEBUG	
	printf("Length of the file \"%s\": %ld\n", check_file, konum);
#endif
	
	
	if( (pid=fork())<0) {
		syslog(LOG_ERR, "main fork() error %m");
		exit(1);// for child process pid=0, for main process pid=number
	}else if( pid ) exit(0); //main process ends, child is going to the background
	
	//Starts the program in a new session
	setsid();
	
	(void)signal(SIGINT, cik);
	(void)signal(SIGTERM, cik);
	(void)signal(SIGHUP, cik);
	(void)signal(SIGQUIT, cik);
	
	while(1) {
		// This will inform you at predefined times, that the prog is running without any problem.
		time(&curtime);	tmb=localtime(&curtime);
		if( (tmb->tm_hour==TIME1 || tmb->tm_hour==TIME2) && report ) {
			sprintf(buf1, "%s is running.\nIt's looking for strings below in the file \"%s\":\n", argv[0], check_file);
			for(i=0; i<satir_sayisi; i+=2) {
				sprintf(buf2, "string[%d]: %s\n", i/2, satirlar[i]);
				strcat(buf1, buf2);	
			}
#ifdef DEBUG
			syslog(LOG_INFO, buf1);
#endif
			subject_to(satirlar[1], subject, to);
			simple_sendmail(SMTPHOST, FROM, to, OKSUBJECT, buf1);
			report=0; //Do not send this report again
		}			
		// You may send the report if ...
		if( !report && (tmb->tm_hour != TIME1  && tmb->tm_hour != TIME2)  ) report=1;

		
		sleep(2);
		
		if( !(dosya=fopen(check_file, "r")) ) {
			konum=0;
			continue; // The file has been removed
		}
		fseek(dosya, 0, SEEK_END);
		file_length=ftell(dosya);
		
		if( file_length == konum ) {
			fclose(dosya);	
			continue; // There isn't any change.
		}
		// If file_length < konum this means that the file has just been recreated
		// so, start from the beginning of the file
		konum=file_length > konum ? konum : 0L;
		
		fseek(dosya, konum, SEEK_SET);
		
		while( fgets(message, MAXUZ, dosya) ) {
			puts(message);	
			for(j=0; j<satir_sayisi; j+=2)
				if ( !fnmatch(satirlar[j], message, 0) ) {
#ifdef DEBUG					
					printf("Bulundu:%s\n", message);
#endif
					if( satirlar[j][0]=='^' ) //Ignore the lines if they contain that string
						if( !fnmatch(&satirlar[j][1], message, 0) ) break;
					subject_to(satirlar[j+1], subject, to);
					comma(message); //convert commas to newlines, for readability
					simple_sendmail(SMTPHOST, FROM, to, subject, message);
					break; // Do not look at any other pattern
				}
		}
		
		konum=ftell(dosya);
#ifdef DEBUG		
		printf("Current length of the file: %ld\n", konum);
#endif
		fclose(dosya);
	}
}

void comma(char *buf)
{
	int i=-1; 
	while( buf[++i]!='\n' )
		if( buf[i]==',') buf[i]='\n'; 
}
int subject_to(char *string, char *subject, char *to)
{
	int i=0;
	while( (subject[i]=string[i++])!='|' );
	subject[i-1]=(char)NULL;
	strncpy(to, &string[i], MAXSTR);
	if( !strstr(to, "@") ) {
		printf("E-mail adresi icinde @ yok:%s !\n", to);
		exit(1);
	}
	return 0;
}

void cik(int sig)
{
	int i;
		
	syslog(LOG_INFO, "Ending.");
	printf("\nEnding: %d\n", sig);
	for(i=0; i<satir_sayisi; i++) 
		free(satirlar[i]);
	free(satirlar);
	exit(0);
}
